import sys

class InputControl:

    def concat_words(self, command_words, command_position):
        '''
        Invokes on 'concat'.
        Can concat multiple words or characters depending on a skip count. 

        '''
        command_words[command_position] = command_words[command_position] + command_words[command_position+1]
        del command_words[command_position+1]

    def upper_word(self, command_words, command_position):
        '''
        Invokes on 'upper'.
        Uppercases the whole word.
        '''
        command_words.insert(command_position, command_words[command_position].upper())
        del command_words[command_position+1]
        command_words.insert(len(command_words), '')

    def lower_word(self, command_words, command_position):
        '''
        Invokes on 'lower'.
        Lowercases the whole word.
        '''
        command_words.insert(command_position, command_words[command_position].lower())
        del command_words[command_position+1]
        command_words.insert(len(command_words), '')
    
    def capital_word(self, command_words, command_position):
        '''
        Invokes on 'capital'.
        Capitalizes the next word.
        If the first letter is already Uppercased,
        function reinserts the same word.
        '''
        command_words.insert(command_position, command_words[command_position].capitalize())
        del command_words[command_position+1]
        command_words.insert(len(command_words), '')

    def tilde(self, command_words, command_position):
        '''
        Invokes on 'tilde'.
        Adds tilde (~) character to the input.
        '''
        command_words.insert(command_position, '~')
        self.output_control.print(self.output_control.CHAR_ADDED, ('~', command_position))

    def slash(self, command_words, command_position):
        '''
        Invokes on 'slash'.
        Adds slash (/) character to the input.
        '''
        command_words.insert(command_position, '/')
        self.output_control.print(self.output_control.CHAR_ADDED, ('/', command_position))
    
    def multiply_words(self, command_words, command_position):
        '''
        Invokes in 'multiply'.
        Multiplication of the next 2 words.
        Tries to convert the input to floats and return their product.
        '''
        try:
            command_words[command_position] = float(command_words[command_position]) * float(command_words[command_position+1])
        except:
            pass

    def ignore_skip(self, command_words, command_position):
        '''
        Invokes on 'ignore'.
        Deactivates the rest of the words under the pre-activated skip.
        '''
        pass

    def skip(self, command_words, command_position):
        '''
        Invokes on "skip skip". 
        As the first skip is removed before the invokation, 
        the second is being ignored.

        '''
        pass
        

    hardcoded_dict = {
    'concat':concat_words,
    'upper':upper_word,
    'lower':lower_word,
    'capital':capital_word,
    'tilde':tilde,
    'slash':slash,
    'multiply':multiply_words,
    'ignore':ignore_skip,
    'skip':skip
    }

    def format_input(self, input_string):
        '''
        Input is being re-written by the usage of the "skip" variable.
        1) To write "skip", say the word twice: "skip skip"
        2) To concat multiple words/letters, say "skip [Number of the words/letters] [first_word] [second_word]..."
        3) To write a special character like %, say "skip percent"
        4) To concat this character with another word/letter, use it inside "skip INTEGER ...",
            For example, to write "10%" say "skip 2 10 percent"
        5) upper, lower, capital and other word modifiers need to be inside "skip", but they don't take a number of words/letters.
            Output of "skip 5 capital downloads slash hello . txt" is "Downloads/hello.txt"
            Output of "skip 5 downloads slash hello . txt" is "downloads/hello.txt"
            Output of "skip 6 capital downloads slash hello . txt" should be an error, - too few words. 
            Output of "skip 4 downloads slash hello . txt" should be an error, - too many words.
        '''

        try:
            oc = self.output_control
            cc  = input_string.rstrip().split(" ")
            oc.print(oc.BFR_COM_WORDS, cc)
            clear_position = 0
            command_count = 0
            while clear_position < len(cc):
                hardcoded_commands = []
                concat_tuple = ()
                if cc[clear_position] == 'skip':
                    del cc[clear_position]

                    if cc[clear_position] == 'skip':
                        pass

                    elif cc[clear_position] in self.hardcoded_dict.keys():
                        hardcoded_command = self.hardcoded_dict[cc[clear_position]]
                        del cc[clear_position]
                        hardcoded_command(self, cc, clear_position)

                    elif (cc[clear_position]).isdigit():
                        command_count = int(cc[clear_position])
                        del cc[clear_position]

                        for command_index in range(clear_position, clear_position + command_count):
                            if cc[command_index] in self.hardcoded_dict:
                                hardcoded_command = self.hardcoded_dict[cc[command_index]]
                                del cc[command_index]
                                hardcoded_command(self, cc, command_index)

                        command_index = 0
                        while command_index < command_count-1:
                            self.concat_words(cc, clear_position)
                            command_index +=1

                    else:
                        print('Command not found, or "skip" at the end.')
                        break
                clear_position += 1
            cc = list(filter(None, cc))
            return (cc)
        except:
            pass

    def __init__(self, output_control):
        self.output_control = output_control



import sqlite3
#import json

#LOCAL FILES
from .queries import QueryList
from .data_types.language import Language
from .data_types.module import Module
from .data_types.command import Command
from .data_types.external_module import ExternalModule
from .data_types.imprt import Imprt
from .data_types.call import Call 
from .data_types.word import Word
from .data_types.combo import Combo

class Database:
    skc_level = 0
    def create_structure(self):
        '''
        Creates a database, if one's structure is not already created.
        It is called on initial app start, or if database does not have a proper format.
        '''
        structure_queries_text = (
                self.query_list.create_languages.text,
                self.query_list.create_modules.text,
                self.query_list.create_external_modules.text,
                self.query_list.create_commands.text,
                self.query_list.create_imports.text,
                self.query_list.create_calls.text,
                self.query_list.create_words.text,
                self.query_list.create_combos.text)
        for structure_query_text in structure_queries_text:
            self.cursor.execute(structure_query_text)

    def insert_language_data(self, languages_dict):
        '''
        Initiates new Languages objects,
        populates their values from the languages_dict parameter,
        and saves them to the database.
        '''
        for language_data in languages_dict['languages']:
            language = Language(self.query_list,
                    cursor = self.cursor,
                    code = language_data['code'],
                    name = language_data['name'],
                    active = language_data['active'],
                    output_control = self.output_control)
            language.insert()

    def insert_command_data(self, command_data, module_uid):
        '''
        Initiates new Command, ExternalModules, Imprts, Calls, Words, Combos objects,
        populates their values from the command_data adn module_uid parameters,
        and saves them to the database.
        '''
        oc = self.output_control
        command = Command(self.query_list,
                cursor = self.cursor,
                module_id = module_uid,
                name = command_data['name'],
                code = command_data['code'],
                programming_language = command_data['programming_language'],
                script_url = command_data['script_url'],
                definition = command_data['definition'],
                description = command_data['description'],
                active = command_data['active'],
                output_control = self.output_control)
        command.insert()
        oc.print(oc.COM_OK, (command.name,))
        for external_module_data in command_data['external_modules']:
            external_module = ExternalModule(self.query_list,
                    cursor = self.cursor,
                    name = external_module_data['name'],
                    active = external_module_data['active'],
                    output_control = self.output_control)
            external_module.insert()
            oc.print(oc.EXM_OK, (external_module.name,))
            imprt = Imprt(self.query_list,
                    cursor = self.cursor,
                    command_id = command.uid,
                    external_module_id = external_module.uid,
                    output_control = self.output_control)
            imprt.insert()
            oc.print(oc.IMPRT_OK, (external_module.name, command.name))
        for call_data in command_data['calls']:
            if len(call_data['words'])>self.skc_level:
                self.skc_level = len(call_data['words'])
            call = Call(self.query_list,
                    cursor = self.cursor,
                    command_id = command.uid,
                    language_code = call_data['language'],
                    response = call_data['response'],
                    active = call_data['active'],
                    output_control = self.output_control)
            call.insert()
            for word_data in call_data['words']:
                word = Word(self.query_list,
                        cursor = self.cursor,
                        text = word_data['text'],
                        active = call.active,
                        output_control = self.output_control)
                word.insert()
                combo = Combo(self.query_list,
                        cursor = self.cursor,
                        call_id = call.uid,
                        word_id = word.uid,
                        variable_length = word_data['variable_length'],
                        optional = word_data['optional'],
                        position = word_data['position'],
                        output_control = self.output_control)
                combo.insert()

    def insert_module_data(self, module_dict):
        '''
        Initiates a new Module object,
        populates it's values from the module_dict parameter,
        saves it to the database,
        and calls Database.insert_command_data() on the same database.
        '''
        module_meta = module_dict['module']['metadata']
        module = Module(self.query_list,
                cursor = self.cursor,
                name = module_meta['name'],
                code = module_meta['code'],
                description = module_meta['description'],
                active = module_meta['active'],
                output_control = self.output_control)
        module.insert()
        for command_data in module_dict['module']['commands']:
            self.insert_command_data(command_data, module.uid)

    def __init__(self, output_control, file_path, languages_dict, module_dicts, command_list):
        self.output_control = output_control
        self.connection = sqlite3.connect(file_path)
        self.cursor = self.connection.cursor()
        self.query_list = QueryList()
        self.create_structure()
        self.insert_language_data(languages_dict)
        for module_dict  in module_dicts:
            self.insert_module_data(module_dict)

        for command_dicts in command_list:
            for command_dict in command_dicts:
                #self.output_control.print(self.output_control.COM_DICT, (json.dumps(command_dict, sort_keys=True, indent=2),))
                module_uid = self.cursor.execute(self.query_list.select_module_by_name.text, (command_dict['command']['module_name'],)).fetchone()
                self.insert_command_data(command_dict['command'], module_uid[0])

        self.query_list.construct_skc_query(self.skc_level)
        self.output_control.print(self.output_control.QUERY_CONSTR, (self.skc_level,))
        #self.connection.commit()



class ExternalModule:
    def set(self, uid, name, active):
        '''
        Browse the database for existing external module.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        ext_module_in_db = self.cursor.execute(self.query_list.select_external_module.text, (uid, name)).fetchone()
        if ext_module_in_db is not None:
            self.uid = ext_module_in_db[0]
            self.name = ext_module_in_db[1]
            self.active = ext_module_in_db[2]
            self.output_control.print(self.output_control.DATA_SET, ('External module', 'ID: ', self.uid, ', Name: ', self.name,'',''))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('External module', 'ID: ', self.uid, ', Name: ', self.name,'',''))

    def insert(self):
        '''
        Browse the database for existing external module.
        1) If not found, saves the external module to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        ext_module_in_db = self.cursor.execute(self.query_list.select_external_module.text, ('%', self.name)).fetchone()
        if ext_module_in_db is None:
            self.cursor.execute(self.query_list.insert_external_module.text, (self.name, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('External module', 'ID: ', self.uid, ', Name: ', self.name,'',''))       
        else:
            self.uid = ext_module_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('External module', 'ID: ', self.uid, ', Name: ', self.name,'',''))    
            
    def __init__(self, query_list = None, cursor=None, uid='%', name='%', active = True, output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.name = name
        self.active = active
        self.output_control = output_control



import os
import json
from pkg_resources import resource_exists, resource_string

class FileHandler:
    def load_from_path(self, file_path):
        oc = self.output_control
        if os.path.exists(file_path):
            try:
                json_file = open(file_path, 'r', encoding='utf-8-sig')
                content = json.load(json_file)
                json_file.close()
                return content 
            except:
                oc.print(oc.COULD_NOT_LOAD, (file_path,))
        else:
            oc.print(oc.SETT_FILE_NOT_FOUND, (file_path,))

    def __init__(self, output_control):
        self.output_control = output_control

    def read_library(self, file_path):
        oc = self.output_control
        oc.print(oc.FILE_PATH, (file_path,))
        data_dicts = []
        directories = []
        for file_name in os.listdir(file_path):
            oc.print(oc.FILE_CHECK, (file_name,))
            if '.json' in file_name:
                data_dicts.append(self.load_from_path('{0}{1}'.format(file_path, file_name)))
                oc.print(oc.FILE_ADDED_COMS)
            elif '.' in file_name:
                oc.print(oc.WRONG_EXT)
            else:
                oc.print(oc.IS_DIR, (file_name,))
                directories.append(file_name)
        return (data_dicts, directories)



import io
import os

from google.cloud import speech
from google.cloud.speech import enums
from google.cloud.speech import types

class GoogleTranscriber:
    client = speech.SpeechClient()

    def transcribe(self, audio_path):
        file_name = os.path.join(os.path.curdir, audio_path)
        self.output_control.print(self.output_control.TRANSC, (audio_path,))
        with io.open(file_name, 'rb') as audio_file:
            audio = types.RecognitionAudio(content=audio_file.read())
        response = self.client.recognize(self.config, audio)
        return response.results[0].alternatives

    def __init__(self, output_control, audio_settings_dict, expected_calls):
        self.output_control = output_control
        for audio_values in audio_settings_dict['audio_values']:
            if audio_values['active'] == 'True':
                self.config = types.RecognitionConfig(
                    encoding = enums.RecognitionConfig.AudioEncoding.LINEAR16,
                    sample_rate_hertz = int(audio_values['rate']),
                    language_code = audio_values['language'],
                    max_alternatives = int(audio_values['max_alternatives']),
                    speech_contexts = [{ "phrases": expected_calls}]
                    )



#!/usr/bin/env python3
import os
import sys

from .brain.db_memory.database import Database
from .brain.fs_memory.file_handler import FileHandler
from .brain.processing.command_finder import CommandFinder
from .brain.processing.executor import Executor
from .ears.listener import Listener
from .ears.input_control import InputControl
from .antenna.google_transcriber import GoogleTranscriber
from .antenna.bing_transcriber import BingTranscriber

class PeakBot:
    module_dicts = []
    command_list = []
    database_path = ""
    def set_database_path(self):
        '''
        Function for extracting a database path.
        Assumes only one database is active.
        '''
        oc = self.output_control
        oc.print(oc.DB_PATH_SET_ATT)
        try:
            if 'databases' in self.settings_dict:
                for database_data in self.settings_dict['databases']:
                    if database_data['database_active'] == 'True':
                        if database_data['database_engine'] == 'sqlite3':
                            self.database_path = ('{0}{1}'.format(database_data['database_dir'], database_data['database_filename']))
                            oc.print(oc.USING_DB, (database_data['database_filename'],))
                            break
                        elif database_data['database_engine'] in ('MS SQL Server', 'MySql', 'MySql/MariaDB', 'MongoDB'):
                            oc.print(oc.ENGINE_NOT_SUP, (database_data['database_engine'],))
                        else:
                            oc.print(oc.UNKNOWN_ENGINE, (database_data['database_engine'],))
                    else:
                        oc.print(oc.INACTIVE_DB, (database_data['database_filename']),)
                oc.print(oc.DB_PATH_SET)
        except Exception as e:
            oc.print(oc.DB_PATH_NOT_SET, (str(e),))

    def set_modules_and_commands(self, library_path):
        '''
        This function is looking for modules and commands in the defined directory.
        'get_modules_and_commands' is using 'FileHandler' class, and it's 'check_path' function.
        '''
        oc = self.output_control
        oc.print(oc.MOD_COM_SET_ATT)

        (self.module_dicts, self.directories) = self.file_handler.read_library(library_path) 
        print('GOT MODULE DICT: \n' + str(self.module_dicts))
        print('\n \n GOT DIRECTORIES: \n' + str(self.directories))
        try:
            for directory in self.directories:
                print('\n DIRECTORY CHECK: \n' + str(directory))
                if ('{0}.json'.format(directory)) in os.listdir(library_path):
                    oc.print(oc.JSON_EXISTS, (directory, library_path, directory))
                    (command_dict, self.skip_directories) = self.file_handler.read_library('{0}{1}/'.format(library_path, directory))
                    self.command_list.append(command_dict)
                    oc.print(oc.DIR_WITH_COMS, (directory,))
                    oc.print(oc.SKIP_DIRS, (self.skip_directories,))
                else:
                    print('\n ITS DIRECTORY...: \n')
                    self.skip_directories.append(directory)
                    self.directories.remove(directory)
                    oc.print(oc.NO_JSON_FILE, (directory,))
            oc.print(oc.MOD_COM_SET)
        except Exception as e:
            oc.print(oc.MOD_COM_NOT_SET, (str(e),))
                
    def init_database(self):
        '''
        Initiates a new database.
        '''
        self.output_control.print(self.output_control.INIT_ATT, ('database',))
        #try:
        if True:
            self.database = Database(self.output_control, self.database_path, self.languages_dict, self.module_dicts, self.command_list)
            self.output_control.print(self.output_control.INIT, ('Database',))
        else:
            e=''
        #except Exception as e:
            self.output_control.print(self.output_control.NOT_INIT, ('Database', str(e)))

    def init_listener(self, audio_wav_path):
        '''
        Initiates a new listener.
        '''
        self.output_control.print(self.output_control.INIT_ATT, ('listener',))
        try:
            self.listener = Listener(self.output_control, self.audio_settings_dict, audio_wav_path)
            self.output_control.print(self.output_control.INIT, ('Listener',))
        except Exception as e:
            self.output_control.print(self.output_control.NOT_INIT, ('Listener', str(e)))
            sys.exit()

    def init_command_finder(self):
        '''
        Initiates a new command_finder.
        '''
        self.output_control.print(self.output_control.INIT_ATT, ('command finder',))
        try:
            self.command_finder = CommandFinder(self.output_control, self.input_control, self.database)
            self.output_control.print(self.output_control.INIT, ('Command finder',))
        except Exception as e:
            self.output_control.print(self.output_control.NOT_INIT, ('Command finder', str(e)))
            sys.exit()

    def init_transcriber(self, expected_calls):
        '''
        Initiates a new transcriber.

        '''
        speech_apis_dict = {
                'GOOGL': GoogleTranscriber,
                'BING': BingTranscriber
                }
        self.output_control.print(self.output_control.INIT_ATT, ('transcriber',))
        try:
            for speech_api in self.settings_dict['speech_apis']:
                if speech_api['active']:
                    transcriber_type = speech_apis_dict[speech_api['code']]
                    break
            
            self.transcriber = transcriber_type(self.output_control, self.audio_settings_dict, expected_calls)
            self.output_control.print(self.output_control.INIT, ('Transcriber',))
        except Exception as e:
            self.output_control.print(self.output_control.TRANSC_NOT_INIT, ('Transcriber', str(e)))
            sys.exit()


    def init_executor(self):
        '''
        Initiates a new executor.
        '''
        self.output_control.print(self.output_control.INIT_ATT, ('executor',))
        try:
            self.executor = Executor(self.output_control, self.database)
            self.output_control.print(self.output_control.INIT, ('Executor',))
        except Exception as e:
            self.output_control.print(self.output_control.NOT_INIT, ('Executor', str(e),))
            sys.exit()

        #RESPONSE INDEX IS THE NUMBER OF ALREADY INPUTED ARGUMENTS (BY INITIAL CALL) - RARELY 0!!! 
    def get_additional_args(self, response_index):
        additional_args = ()
        noninitial_responses = self.database.cursor.execute(self.database.query_list.select_responses_by_command_id.text, (str(self.command_finder.command_id), 26, 50)).fetchall()
        if not noninitial_responses:
            noninitial_responses = ''
        while response_index < len(noninitial_responses):
            response_number = noninitial_responses[response_index][0]
            response_text = noninitial_responses[response_index][1]

            #This was COUNT(word.id) .... ????
            #number_of_words = noninitial_responses[response_index][2]

            '''
            expected_answers=()
            if number_of_words>1:
                for words_index in range(number_of_words):
                    expected_answers = expected_answers + (noninitial_responses[response_index + words_index][1],)
                response_index += number_of_words
            '''

            response_index += 1
            self.listener.record()
            #if len(expected_answers) > 0:
                #send to transcriber as expected words!!!

            alternatives = self.transcriber.transcribe(self.listener.file_path)
            response = self.input_control.format_input(alternatives[0].transcript)
            additional_args = additional_args + (response[0],)
        return additional_args

    def run_peak_bot(self):
        self.exit = False
        while not self.exit:
            self.listener.record()
            alternatives = self.transcriber.transcribe(self.listener.file_path)
            transcript = ''

            '''
            Higher confidence:
            confidence = 0.3
            for alternative in alternatives:
                if alternative.confidence > confidence:
                    transcript = alternative.transcript
                    confidence = alternative.confidence

            By index:
            '''
            for alternative in alternatives:
                if alternative.confidence>0.9:
                    transcript = alternative.transcript
                    break

            self.output_control.print(self.output_control.BFR_COM_WORDS, (transcript,))
            response = self.input_control.format_input(transcript)
            self.output_control.print(self.output_control.AFT_COM_WORDS, (response,))
            self.command_finder.find_commands(response)
            args = self.command_finder.command_args
            args = args + self.get_additional_args(len(args))

            self.executor.execute_command(self.command_finder.command_id, args)
            self.database.connection.commit()
            self.exit = True

    def __init__(self, fundamental_directories, output_control):
        self.output_control = output_control
        oc = output_control
        self.file_handler = FileHandler(output_control)
        self.settings_dict = self.file_handler.load_from_path(fundamental_directories[0])
        self.audio_settings_dict = self.file_handler.load_from_path(fundamental_directories[1])  
        self.languages_dict = self.file_handler.load_from_path(fundamental_directories[2])
        self.input_control = InputControl(output_control)
        self.set_database_path()
        self.set_modules_and_commands(fundamental_directories[3])

        self.init_database()
        self.init_listener(fundamental_directories[4])
        self.init_command_finder()
        self.init_transcriber(self.command_finder.expected_calls)
        self.init_executor()

        self.run_peak_bot()

        self.database.connection.close()
        self.output_control.print(self.output_control.DB_CON_CLOSED)



class Word:
    def set(self, uid, text, active):
        '''
        Browse the database for existing word.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        word_in_db = self.cursor.execute(self.query_list.select_word.text, (uid, text, active)).fetchone()
        if word_in_db is not None:
            self.uid = word_in_db[0]
            self.text = word_in_db[1]
            self.active = word_in_db[2]
            self.output_control.print(self.output_control.DATA_SET, ('Word ','ID: ', self.uid,' Text: ', self.text,'',''))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Word ', 'ID: ', self.uid,' Text: ', self.text,'',''))

    def insert(self):
        '''
        Browse the database for existing word.
        1) If not found, saves the word to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        word_in_db = self.cursor.execute(self.query_list.select_word.text, ('%', self.text, self.active)).fetchone()
        if word_in_db is None:
            self.cursor.execute(self.query_list.insert_word.text, (self.text, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Word ', 'ID: ', self.uid,' Text: ', self.text,'',''))
        else:
            #if self.text == 'var':
            self.uid = word_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Word ', 'ID: ', self.uid,' Text: ', self.text,'',''))

    def __init__(self, query_list = None, cursor = None, uid = '%', text = '%', active = True, output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.text = text
        self.active = active
        self.output_control = output_control



#!/usr/bin/env python3
import os
import sys

from pkg_resources import resource_filename

from .tongue.output_control import OutputControl
from .peak_bot import PeakBot

def main():
    '''
    Sets the verbosity in respect to the user input.
    Checks if settings are good.
    Instantiate a peak-bot.
    '''
    supported_platforms = ('windows', 'linux')
    if sys.platform not in supported_platforms:
        oc.print(oc.PLAT_NOT_SUP, sys.platform) 
        sys.exit()

    verbosity = 0
    if len(sys.argv) > 1:
        if sys.argv[1].isdigit():
            verbosity = sys.argv[1]

    oc = OutputControl(range(0, 8), str(verbosity))
    oc.print(oc.WELCOME_MSG) 
    settings_path = resource_filename(__name__, "peak_data/configuration/settings.json")
    audio_base_path = resource_filename(__name__, "peak_data/configuration/audio_base.json")
    lang_base_path = resource_filename(__name__, "peak_data/configuration/lang_base.json")
    library_path = os.path.dirname(resource_filename(__name__, "peak_data/library/core.json"))+ '/'
    audio_wav_path = os.path.join(os.path.expanduser("~"), ".temp_recording.wav")
    fundamental_directories = (settings_path, audio_base_path, lang_base_path, library_path, audio_wav_path)
    bot = PeakBot(fundamental_directories, oc)
        
if __name__ == '__main__':
    main()



import sqlite3

class Query:
    def __init__(self, usage, supported_engines, description, text):
        self.usage = usage
        self.supported_engines = supported_engines
        self.description = description
        self.text = text

class QueryList:
    create_languages = Query(1, ('SQLite3', 'MySQL'), 'This table holds the languages of the command calls.', \
        'CREATE TABLE IF NOT EXISTS languages ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        name VARCHAR(32), \
        code VARCHAR(16), \
        active BOOLEAN \
        );')

    create_modules = Query(1, ('SQLite3', 'MySQL'), 'Creates the "modules", a table which holds the names of command chunks.', \
        'CREATE TABLE IF NOT EXISTS modules ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        name VARCHAR(32), \
        code VARCHAR(16), \
        description VARCHAR(1024), \
        active BOOLEAN \
        );')

    create_commands = Query(1, ('SQLite3', 'MySQL'), 'Creates the "commands" table. Multiple commands are binded by one module.', \
        'CREATE TABLE IF NOT EXISTS commands ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        module_id INTEGER NOT NULL, \
        name VARCHAR(32), \
        code VARCHAR(16), \
        programming_language VARCHAR(32), \
        definition VARCHAR(8192), \
        script_url VARCHAR(128), \
        description VARCHAR(1024), \
        active BOOLEAN DEFAULT True, \
        FOREIGN KEY (module_id) REFERENCES modules(id) \
        );')

    create_external_modules = Query(1, ('SQLite3', 'MySQL'), 'Creates table for usage with commands - to be imported at execution', \
        'CREATE TABLE IF NOT EXISTS external_modules ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        name VARCHAR(32), \
        active BOOLEAN \
        );')

    create_imports = Query(1, ('SQLite3', 'MySQL'), 'Table for connecting commands and external modules', \
        'CREATE TABLE IF NOT EXISTS imports ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        external_module_id INTEGER NOT NULL, \
        command_id INTEGER NOT NULL, \
        active BOOLEAN DEFAULT True, \
        FOREIGN KEY (external_module_id) REFERENCES external_modules(id) \
        FOREIGN KEY (command_id) REFERENCES commands(id) \
        );')

    create_calls = Query(1, ('SQLite3', 'MySQL'), 'Creates the "calls" table, used for connecting words into a single command combo - call', \
        'CREATE TABLE IF NOT EXISTS calls ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        command_id INTEGER NOT NULL, \
        language_id INTEGER NOT NULL, \
        response INTEGER DEFAULT 0, \
        active BOOLEAN DEFAULT True, \
        FOREIGN KEY (language_id) REFERENCES languages(id) \
        FOREIGN KEY (command_id) REFERENCES commands(id) \
        );')

    create_words = Query(1, ('SQLite3', 'MySQL'), 'Creates the "words" table.', \
        'CREATE TABLE IF NOT EXISTS words ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        text VARCHAR(128), \
        active BOOLEAN DEFAULT True \
        );')

    create_combos = Query(1, ('SQLite3', 'MySQL'), 'Creates the "combos" table. Used for connecting calls and words.', \
        'CREATE TABLE IF NOT EXISTS combos ( \
        id INTEGER PRIMARY KEY AUTOINCREMENT, \
        call_id INTEGER NOT NULL, \
        word_id INTEGER NOT NULL, \
        variable_length INTEGER DEFAULT 0, \
        optional BOOLEAN DEFAULT False, \
        position INTEGER, \
        FOREIGN KEY (word_id) REFERENCES words(id) \
        FOREIGN KEY (call_id) REFERENCES call(id) \
        );')

    insert_language = Query(3, ('SQLite3', 'MySQL'), 'Inserts a single language.', \
        'INSERT INTO languages \
        (code, name, active) \
        VALUES \
        (?, ?, ?);')

    insert_module = Query(3, ('SQLite3', 'MySQL'), 'Inserts a single module.', \
        'INSERT INTO modules \
        (name, code, description, active) \
        VALUES \
        (?, ?, ?, ?);')

    insert_command = Query(1, ('SQLite3', 'MySQL'), 'Inserts a single command.', \
        'INSERT INTO commands \
        (module_id, name, code, programming_language, definition, script_url, description,  active) \
        VALUES \
        (?, ?, ?, ?, ?, ?, ?, ?);')

    insert_external_module = Query(3, ('SQLite3', 'MySQL'), 'Inserts a single external module.', \
        'INSERT INTO external_modules \
        (name, active) \
        VALUES \
        (?, ?);')

    insert_import = Query(8, ('SQLite3', 'MySQL'), 'Inserts a single command combo.', \
        'INSERT INTO imports \
        (command_id, external_module_id) \
        VALUES \
        (?, ?);')

    insert_call = Query(7, ('SQLite3', 'MySQL'), 'Inserts a single call.', \
        'INSERT INTO calls \
        (command_id, language_id, response, active) \
        VALUES \
        (?, ?, ?, ?);')
    
    insert_word = Query(8, ('SQLite3', 'MySQL'), 'Inserts a single word.', \
        'INSERT INTO words \
        (text, active) \
        VALUES \
        (?, ?);')

    insert_combo = Query(8, ('SQLite3', 'MySQL'), 'Inserts a single command combo.', \
        'INSERT INTO combos \
        (call_id, word_id, variable_length, optional, position) \
        VALUES \
        (?, ?, ?, ?, ?);')

    select_language = Query(4, ('SQLite3', 'MySQL'), 'Selects a language from passed variables.', \
        'SELECT lan.* \
        FROM languages AS lan \
        WHERE lan.id LIKE ? \
        AND lan.code LIKE ? \
        AND lan.name LIKE ? \
        AND lan.active == ?;')

    select_module = Query(4, ('SQLite3', 'MySQL'), 'Selects a single module', \
        'SELECT mod.* \
        FROM modules AS mod \
        WHERE mod.id LIKE ? \
        AND mod.name LIKE ? \
        AND mod.code LIKE ? \
        AND mod.active == "True";')

    select_external_module = Query(4, ('SQLite3', 'MySQL'), 'Selects a single external module', \
        'SELECT em.* \
        FROM external_modules AS em \
        WHERE em.id LIKE ? \
        AND em.name LIKE ? \
        AND em.active == "True";')

    select_external_modules_by_command_id = Query(10, ('SQLite3', 'MySQL'), 'Used for collecting modules before import', ' \
        SELECT em.name AS nameo \
        FROM imports AS im \
        LEFT JOIN external_modules AS em ON em.id = im.external_module_id \
        WHERE im.command_id = ?;')


    select_module_by_name = Query(2, ('SQLite3', 'MySQL'), 'Gets the modules uid.', ' \
        SELECT mo.id AS module_id \
        FROM modules AS mo \
        WHERE mo.name = ?;')

    select_import = Query(8, ('SQLite3', 'MySQL'), 'Selects an import ', \
        'SELECT im.* \
        FROM imports AS im \
        WHERE im.id LIKE ? \
        AND im.command_id LIKE ? \
        AND im.external_module_id LIKE ? \
        AND im.active == "True";')

    select_command = Query(8, ('SQLite3', 'MySQL'), 'Selects a command from db', \
        'SELECT com.* \
        FROM commands  AS com \
        WHERE com.id LIKE ? \
        AND com.module_id LIKE ? \
        AND com.name LIKE ? \
        AND com.code LIKE ? \
        AND com.programming_language LIKE ? \
        AND com.active == "True";')

    select_command_by_id = Query(10, ('SQLite3', 'MySQL'), 'Used for finding a single command with provided id', ' \
        SELECT co.name AS name, co.programming_language AS language, co.definition AS definition \
        FROM commands AS co \
        WHERE co.id = ?;')

    select_combo = Query(8, ('SQLite3', 'MySQL'), 'Selects a combo from db', \
        'SELECT cmb.* \
        FROM combos AS cmb \
        WHERE cmb.id LIKE ? \
        AND cmb.call_id LIKE ? \
        AND cmb.word_id LIKE ? \
        AND cmb.position LIKE ?;')

    select_call = Query(8, ('SQLite3', 'MySQL'), 'Selects a call from db', \
        'SELECT cal.* \
        FROM calls AS cal \
        WHERE cal.id LIKE ? \
        AND cal.command_id LIKE ? \
        AND cal.language_id LIKE ? \
        AND cal.response LIKE ? \
        AND cal.active == ?;')

    select_word = Query(8, ('SQLite3', 'MySQL'), 'Selects a word from db', \
        'SELECT wor.* \
        FROM words AS wor \
        WHERE wor.id LIKE ? \
        AND wor.text LIKE ? \
        AND wor.active == ?;')

    select_words_by_position = Query(10, ('SQLite3', 'MySQL'), 'Gets the combinations of words.', ' \
        SELECT w.text \
        FROM command AS com \
        LEFT JOIN combos AS ca ON ca.command_id = co.id \
        LEFT JOIN words AS w ON ca.word_id = w.id \
        WHERE ca.position = ?, co.module_id = ? AND w.language_id = ?;')

    select_responses_by_command_id = Query(10, ('SQLite3', 'MySQL'), 'Used for collecting responses with a recognized command', ' \
        SELECT resp.response, wo.text \
        FROM calls AS resp \
        LEFT JOIN commands AS co ON resp.command_id = co.id \
        LEFT JOIN combos AS cb ON cb.call_id = resp.id \
        LEFT JOIN words AS wo ON wo.id = cb.word_id \
        LEFT JOIN languages AS lan on resp.language_id = lan.id \
        WHERE co.id = ? AND resp.response BETWEEN ? AND ? AND lan.active = "True" \
        ORDER BY resp.response;')

    select_known_calls = Query(10, ('SQLite3', 'MySQL'), 'Used for collecting pre-recognition calls', '    SELECT\n')

    def construct_skc_query(self, number_of_calls):
        '''
        Updates "select_known_calls" query to return exact number of calls.
        '''
        for word_index in range(1, number_of_calls+1):
            self.select_known_calls.text = '{0}    wo{1}.text, cb{1}.variable_length,\n'.format(self.select_known_calls.text, str(word_index))

        self.select_known_calls.text = '{0}    co.id\n\
    FROM modules AS mod\n\
    LEFT JOIN commands AS co ON co.module_id = mod.id \n\
    LEFT JOIN calls AS cal ON cal.command_id = co.id AND cal.response BETWEEN 0 AND 25 \n\
    LEFT JOIN languages AS lan ON lan.id = cal.language_id \n'.format(self.select_known_calls.text)

        for word_index in range(1, number_of_calls+1):
            self.select_known_calls.text = '{0}    LEFT JOIN combos AS cb{1} ON cb{1}.call_id = cal.id AND cb{1}.position == {1}\n\
    LEFT JOIN words AS wo{1} ON wo{1}.id = cb{1}.word_id\n'.format(self.select_known_calls.text, str(word_index))

        self.select_known_calls.text = '{0}    WHERE mod.active = "True" AND lan.active = "True" \n\
        GROUP BY cal.id, wo1.text;'.format(self.select_known_calls.text)




from sys import byteorder
from array import array
from struct import pack

import pyaudio
import wave


class Listener:
    portaudio_formats = {
        'uint8':pyaudio.paUInt8,
        'int8' :pyaudio.paInt8,
        'int16':pyaudio.paInt16,
        'int24':pyaudio.paInt24,
        'int32':pyaudio.paInt32,
        'float32':pyaudio.paFloat32
        }

    def set_values(self, audio_settings_dict):
        for audio_values in audio_settings_dict['audio_values']:
            if audio_values['active'] == 'True':
                self.format = self.portaudio_formats[audio_values['format']]
                self.threshold = int(audio_values['threshold'])
                self.sample_rate = int(audio_values['rate'])
                self.chunk_size = int(audio_values['chunk_size'])
                self.max_volume = int(audio_values['max_volume'])

    def listen(self):
        py_audio = pyaudio.PyAudio()
        py_stream = py_audio.open(format=self.format, channels=1, rate=self.sample_rate, input=True, output=True, frames_per_buffer=self.chunk_size)
        silent_time = 0
        voice_started = False
        audio_data = array('h')

        self.output_control.print(self.output_control.EARS_READY)
        while True:
            temp_data = array('h', py_stream.read(self.chunk_size))
            if byteorder == 'big':
                temp_data.byteswap()
            audio_data.extend(temp_data)

            if not voice_started and max(temp_data) >= self.threshold:
                self.output_control.print(self.output_control.VOICE_DTC)
                voice_started = True
            if voice_started:
                if max(temp_data) < self.threshold:
                    silent_time += 1
                    if silent_time < 25:
                        self.output_control.print(self.output_control.SILENCE, (silent_time,))
                    else:
                        break
                else:
                    silent_time = 0

        sample_width = py_audio.get_sample_size(self.format)
        py_stream.stop_stream()
        py_stream.close()
        py_audio.terminate()
        multiplier = float(self.max_volume)/max(abs(piece) for piece in audio_data)
        temp_data = array('h')
        for piece in audio_data:
            temp_data.append(int(piece*multiplier))
        audio_data = temp_data
        return sample_width, audio_data 

    def record(self):
        sample_width, audio_data = self.listen()
        data = pack('<' + ('h'*len(audio_data)), *audio_data)
        wave_file = wave.open(self.file_path, 'wb')
        wave_file.setnchannels(1)
        wave_file.setsampwidth(sample_width)
        wave_file.setframerate(self.sample_rate)
        wave_file.writeframes(audio_data)
        wave_file.close()

    def __init__(self, output_control, audio_settings_dict, audio_wav_path):
        self.output_control = output_control
        self.set_values(audio_settings_dict)
        self.file_path = audio_wav_path



import json
import os

#from pocketsphinx.pocketsphinx import Decoder

class CommandFinder:
    command_id = 0
    command_args = ()
    command_args_list = []
    check_calls = []
    passed_calls = []
    expected_calls = []

    def set_calls(self):
        '''
        Retreives command calls from the database.
        These calls are, later on, used for comparison with user input.
        Argument "sic_level" determines the number of words to be retreived 
        per command call. It should always be equal to the number 
        of words in the longest command call.
        '''
        #Select calls that are already written in the database: Only active, non-response (response = 0) commands.
        known_calls = (self.database.cursor.execute(self.database.query_list.select_known_calls.text)).fetchall()
        #Clean the variables
        self.check_calls = [] 
        current_call_text = ""
        current_call_vars = [] 
        for known_call in known_calls:
            call_id = known_call[len(known_call)-1]
            for position_in_tuple in range(0, len(known_call)-1, 2):
                call_word = known_call[position_in_tuple]
                call_variable_length = known_call[position_in_tuple+1]
                if call_word is not None:
                    if call_variable_length == 0:
                        current_call_text = current_call_text + known_call[position_in_tuple] + " "
                    elif call_variable_length > 0:
                        current_call_vars.append((int(position_in_tuple/2+1), call_variable_length))

            self.check_calls.append((current_call_text[:-1], tuple(current_call_vars), call_id))

            if current_call_text[:-1] not in self.expected_calls:
                self.expected_calls.append(current_call_text[:-1])
            current_call_text = ""
            current_call_vars = []
        self.output_control.print(self.output_control.EXPECT_CALLS, (self.expected_calls,))

    def __init__(self, output_control, input_control, database):
        self.output_control = output_control
        self.input_control = input_control
        self.database = database
        self.expected_calls = list(self.input_control.hardcoded_dict.keys())
        self.set_calls()

    def get_skip_positions(self, call_variables, check_words):
        oc = self.output_control
        total_index = 0 
        skip_positions = ()
        for variable in call_variables:
            var_position = variable[0] + total_index
            oc.print(oc.VAR_POS_EQ, (str(variable[0]), str(total_index), str(var_position)))
            var_length = variable[1]
            for var_index in range(0, var_length):
                oc.print(oc.VAR_INDEX, (str(var_index),))
                oc.print(oc.VAR_POS, (str(var_position),))
                skip_positions = skip_positions + (var_position + var_index,)
                check_words.insert(var_position + var_index + 1, 'var_space')
                oc.print(oc.VAR_INSTD, (var_position + var_index,))
            total_index = total_index + var_length - 1
            oc.print(oc.TOTAL_INDEX, (str(total_index),))
        oc.print(oc.SKIP_POSS, (str(skip_positions),))
        return skip_positions

    def call_passed(self, command_call, skip_positions, check_words):
        '''
        This function checks if the passed command_call matches the "explicit" users input.
        It's doing so by comparing lengths first and values second.
        This should be changed once the machine learning is implemented.
        '''
        oc = self.output_control
        cc = command_call
        #Check if the number of words are the same.

        if len(cc) == len(check_words):
            clear_index = 0
            for clear_position in range(len(cc)):
                if cc[clear_position] == check_words[clear_position - clear_index]:
                    oc.print(oc.WORD_GOOD, (cc[clear_position], clear_position + 1))

                elif (clear_position+1) in skip_positions:
                    oc.print(oc.PASSD_VAR, (cc[clear_position], clear_position + 1))
                    self.command_args = self.command_args + (cc[clear_position],)
                    clear_index += 1

                else:
                    oc.print(oc.WORD_MISS, (clear_position+1,cc[clear_position], check_words[clear_position-clear_index]))
                    oc.print(oc.POS_NOT_FOUND, (clear_position + 1, str(skip_positions)))
                    oc.print(oc.CALL_DIDNT_PASS)
                    return False
                    break

            if self.command_args != ():
                self.command_args_list.append(self.command_args)
                self.command_args = ()

            return True
        else:
            oc.print(oc.UNEQUAL_WORDS_NO, (len(check_words), len(cc)))
            oc.print(oc.CALL_DIDNT_PASS)
            return False

    def find_commands(self, result):
        oc = self.output_control
        ic = self.input_control
        oc.print(oc.AFT_COM_WORDS, (result,))
        skip_positions = ()
        for check_call in self.check_calls:
            check_words = check_call[0].split(" ")
            check_variables = check_call[1]
            number_of_check_variables = len(check_variables)
            oc.print(oc.SPLITTER)
            oc.print(oc.CHECK_CALL, (check_call,))
            oc.print(oc.SPLITTER)
            oc.print(oc.NO_OF_VARS, (number_of_check_variables,))
            skip_positions = self.get_skip_positions(check_call[1], check_words)
            if self.call_passed(result, skip_positions, check_words):
                self.passed_calls.append(check_call)

        #Check if any calls passed:
        if len(self.passed_calls)>0:

            if len(self.passed_calls)==1:
                self.command_id = self.passed_calls[0][2]
                    
                #Check if there are any passed arguments:
                if len(self.command_args_list)>0:
                    #Take the first (and only) tuple from command_args_list
                    self.command_args = self.command_args_list[0]

                oc.print(oc.RECOGN_COMS)
                for passed_call in self.passed_calls:
                    oc.print(oc.RECOGN_COM_DATA, (passed_call[2], passed_call[0]))

            #If multiple calls passed:
            else:
                for passed_call in self.passed_calls:
                    print(passed_call)
        #If no calls passed:
        else:
            oc.print(oc.NO_COMS_PASSED)
            #Prompt for a creation of new command.
        oc.print(oc.SPEECH_PRC)



class Language:
    def set(self, uid, code, name, active):
        '''
        Browse the database for existing language.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        lang_in_db = self.cursor.execute(self.query_list.select_language.text, (uid, code, name, active)).fetchone()
        if lang_in_db is not None:
            self.uid = lang_in_db[0]
            self.code = lang_in_db[1]
            self.name = lang_in_db[2]
            self.active = lang_in_db[3]
            self.output_control.print(self.output_control.DATA_SET, ('Language','name', self.name,'','','',''))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Language', 'name', self.name,'','','',''))
            print('Language with ID:{0} not found.'.format(self.uid))

    def insert(self):
        '''
        Browse the database for existing language.
        1) If not found, saves the language to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        lang_in_db = self.cursor.execute(self.query_list.select_language.text, (self.uid, self.code, self.name, self.active)).fetchone()
        if lang_in_db is None:
            self.cursor.execute(self.query_list.insert_language.text, (self.code, self.name, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Language', 'name', self.name,'','','',''))
        else:
            self.uid = lang_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Language', 'name', self.name,'','','',''))

    def __init__(self, query_list = None, cursor=None, uid='%', code = '%', name='%', active = True, output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.code = code 
        self.name = name
        self.active = active
        self.output_control = output_control




class Imprt:
    def set(self, uid, command_id, external_module_id):
        '''
        Browse the database for existing import.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        imprt_in_db = self.cursor.execute(self.query_list.select_import.text, (uid, command_id, external_module_id)).fetchone()
        if imprt_in_db is not None:
            self.uid = imprt_in_db[0]
            self.command_id = imprt_in_db[1]
            self.external_module_id = imprt_in_db[2]
            self.output_control.print(self.output_control.DATA_SET, ('Import', 'Command ID: ', self.command_id, ', External Module\'s ID: ', self.external_module_id,'',''))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Import', 'Command ID: ', self.command_id, ', External Module\'s ID: ', self.external_module_id,'',''))

    def insert(self):
        '''
        Browse the database for existing import.
        1) If not found, saves the import to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        imprt_in_db = self.cursor.execute(self.query_list.select_import.text, ('%', self.command_id, self.external_module_id)).fetchone()
        if imprt_in_db is None:
            self.cursor.execute(self.query_list.insert_import.text, (self.command_id, self.external_module_id))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Import', 'Command ID: ', self.command_id, ', External Module\'s ID: ', self.external_module_id,'',''))        
        else:
            self.uid = imprt_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Import', 'Command ID: ', self.command_id, ', External Module\'s ID: ', self.external_module_id,'',''))
    def __init__(self, query_list = None, cursor = None, uid = '%', command_id = '%', external_module_id = '%', output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.command_id = command_id
        self.external_module_id = external_module_id
        self.output_control = output_control



import io
import os

class BingTranscriber:
    def transcribe(self, audio_path):
        pass

    def __init__(self, output_control, audio_settings_dict, expected_calls):
        self.output_control = output_control



import os

class OutputControl:
    #STRING_NAME = ('Some text', output GROUP, VERBOSITY required to print, success SIGN)
    signs = {}
    txt_to_speech = False
    '''
    output_groups:
    main/general = 0
    peak_bot = 1
    file_handler = 2
    database = 3
    listener = 4 
    transcriber = 5
    executioner = 6 
    '''
    verbosity_level = 0
    output_groups = ()
    signs = ('     ', ' >>  ', ' ??  ', ' !!  ', ' **  ', ' xx  ', '  ')
    '''
    Main/General
    output_group: 0
    '''
    WELCOME_MSG = ('Application started.', 0, 3, 1)
    PLAT_NOT_SUP = ('{0} is not supported.', 0, 6, 5)
    SPLITTER = ('-------------------------------', 0, 3, 6)
    '''
    PeakBot class
    output_group: 1 
    '''
    SETT_FILE_NOT_FOUND = ('File {0} not found.', 2, 3, 4)
    USING_DB = ('Using database {0}', 1, 2, 1)
    ENGINE_NOT_SUP = ('Engine {0} is not yet supported. Skipping...', 1, 3, 0)
    UNKNOWN_ENGINE = ('Unknown database engine: {0}. Skipping...', 1, 3, 0)
    INACTIVE_DB = ('Database {0} is not active. Skipping...', 1, 3, 0)
    DIR_WITH_COMS = ('{0} directory contains commands', 1, 3, 1)
    COM_LST = ('Command list: {0}', 1, 5, 0)
    SKIP_DIRS = ('Skip directories: {0}', 1, 4, 0)
    NO_JSON_FILE = ('{0} does not have a coresponding .json file in "/modules"', 1, 3, 4)
    DB_PATH_SET_ATT = ('Attempting to set the path for the database...', 1, 3, 4)
    DB_PATH_SET = ('Database path is set.', 1, 3, 1)
    DB_PATH_NOT_SET = ('Database path could not be set.\nException: {0}', 1, 1, 5)
    JSON_EXISTS = ('Directory {0} have a coresponding file {1}{2}.json', 1, 3, 1)
    MOD_COM_SET_ATT = ('Attempting to set modules and commands...', 1, 3, 4)
    MOD_COM_SET = ('Modules and commands are set.', 1, 3, 1)
    MOD_COM_NOT_SET = ('Modules and commands could not be set.\nException: {0}.', 1, 1, 5)
    INIT_ATT = ('Attempting to initiate the {0}...', 1, 3, 4)
    INIT = ('{0} is initiated.', 1, 3, 1)
    NOT_INIT = ('{0} could not be initiated.\nException: {1}.', 1, 1, 5)
    SPEECH_PRC = ('Speech is processed.', 1, 3, 1)
    COM_EXEC = ('Command is executed.', 1, 3, 1)
    COM_NOT_EXEC = ('Command could not be executed.\nException: {0}.', 1, 1, 5)
    DB_CON_CLOSED = ('Database connection is closed.', 1, 3, 1)
    '''
    FileHandler class
    output_group: 2 
    '''
    COULD_NOT_LOAD = ('Could not load the file {0}.', 2, 3, 4)
    FILE_NOT_FOUND = ('File {0} not found.', 2, 3, 4)
    FILE_PATH = ('File path: {0}.', 2, 4, 0)
    FILE_CHECK = ('Checking if {0} is a file...', 2, 3, 2)
    FILE_ADDED_COMS = ('Content of .json file is added to the list of commands.', 2, 3, 1)
    WRONG_EXT = ('Not .JSON extension. Skipping...', 2, 3, 5)
    IS_DIR = ('{0} is a directory.', 2, 3, 1)
    '''
    Database class
    output_group: 3
    '''
    COM_OK = ('Command {0} in working condidion.', 3, 3, 1)
    EXM_OK = ('External module {0} in working condition.', 3, 3, 1)
    IMPRT_OK = ('Import {0} in working condition {1}.', 3, 3, 1) 
    COM_DICT = ('Command dictionary: {0}', 3, 5, 0)
    QUERY_CONSTR = ('Constructed query with {0} skc level', 3, 3, 1)
    '''
    Listener class
    output_group:4 
    '''
    EARS_READY = ('Ready for input...', 4, 1, 1)
    VOICE_DTC = ('Voice detected. Listening started...', 4, 4, 1)
    SILENCE = ('Silence: {0}/25.', 4, 4, 4)
    '''
    Transcriber class
    output_group: 5
    '''
    TRANSC = ('Transcribing {0}:', 5, 2, 0)

    BFR_COM_WORDS = ('Command words (before formatting): {0}', 5, 4, 0)
    AFT_COM_WORDS = ('Command words (after formatting): {0}', 5, 4, 0)
    CALL_LISTN = ('Listening for a command call...', 5, 1, 4)
    READ_CRED = ('Reading credentials...', 5, 3, 4)
    SEND_AUDIO = ('Uploading audio file...', 5, 3, 4)
    LOCAL_RECOGN = ('Recognizing localy...', 5, 3, 4)
    EXPECT_CALLS = ('Expected calls: {0}.', 5, 5, 1)
    RSTORE_EXPECT_CALLS = ('Restoring expected calls to a bare state...', 5, 3, 4)
    VAR_POS_EQ = ('variable[0]({0}) + total_index({1}) =  var_position({2})', 5, 5, 0)
    VAR_INDEX = ('Variable index: {0}', 5, 4, 0)
    VAR_POS = ('Variable position: {0}', 5, 4, 0)
    VAR_INSTD = ('Inserted variable at position {0}', 5, 3, 1)
    TOTAL_INDEX = ('Total index: {0}', 5, 4, 0)
    CHECK_CALL = ('Check call: {0}', 5, 3, 6)
    NO_OF_VARS = ('Number of variables: {0}', 5, 4, 0)
    SKIP_POSS = ('Skip positions: {0}', 5, 4, 0)
    WORD_GOOD = ('Word {0} at position {1} good, not a variable.', 5, 3, 1)
    CUR_ARGS = ('Current command arguments: {0}', 5, 4, 0)
    PASSD_VAR = ('Passed variable {0} at position {1}.', 5, 3, 1)
    WORD_MISS = ('Word missmatch at position {0}: {1} != {2}.', 5, 3, 4)
    POS_NOT_FOUND = ('Position {0} not found in list of variable positions {1}.', 5, 3, 4)
    UNEQUAL_WORDS_NO = ('This command have {0} words. {1} provided.', 5, 3, 4)
    CALL_PASS = ('Call passed.', 5, 3, 1)
    CALL_DIDNT_PASS = ('Call didn\'t pass.', 5, 3, 5)
    RECOGN_COMS = ('Recognized commands:', 5, 1, 0)
    RECOGN_COM_DATA = ('ID: {0}  Text: {1}', 5, 1, 1)
    NO_COMS_PASSED = ('No commands passed', 5, 1, 5)
    TIMED_OUT = ('Audio read timed out.\nException: {0}', 5, 1, 5)
    COULD_NOT_UNDRS = ('Could not understand audio.\nException: {0}', 5, 1, 5)
    COULD_NOT_TRANS = ('Counld not transmit audio.\nException: {0}', 5, 1, 5)
    '''
    Executor class
    output_group: 6
    '''
    MOD_ATT_IMPORT = ('Attempting to import module "{0}".', 6, 1, 1)
    MOD_IMPORT = ('Module "{0}" imported.', 6, 3, 1)
    MOD_NOT_IMPORT = ('Unable to import the module: "{0}".\nException: {1}.', 6, 1, 5)
    SUC_RESP = ('{0}', 5, 0, 0)
    '''
    Data classes
    output_group: 7
        - Languages
        - Modules
        - External Modules
        - Imports
        - Commands
        - Calls
        - Combos
        - Words
    '''
    DATA_SET = ('{0} with values {1}{2}{3}{4}{5}{6} is set.', 7, 3, 1)
    DATA_NOT_FOUND = ('{0} with values {1}{2}{3}{4}{5}{6} is not found in the database.', 7, 1, 5)
    DATA_INSRT = ('{0} with values {1}{2}{3}{4}{5}{6} is inserted.', 7, 3, 1)
    DATA_EXISTS = ('{0} with values {1}{2}{3}{4}{5}{6} is already in the database.', 7, 3, 1)
    '''
    Input class
    output_group: 8
    '''
    CHAR_ADDED = ('Added "{0}" to position {1}.', 8, 4, 1)
    

    '''
    Output function with 5 level verbosity (0-4).
    Level 5 is comment alike... It never prints.
    '''
    def print(self, output, args=()):
        '''
	Checks if verbosity is larger than potential output.
	Prepares "text" variable to be printed.
	Checks if it shoud print or speak the response.
        '''
        if int(output[2]) <= int(self.verbosity_level) and output[1] in self.output_groups:
            text = output[0].format(*args)
            if self.txt_to_speech:
                pass
            else:
                # To remove tab-signs from command output: 
                # Uncomment:
                #print('{0}'.format(text))
                # Comment out:
                print('{0}{1}'.format(self.signs[output[3]], text))


    def __init__(self, output_groups, verbosity_level):
        self.output_groups = output_groups
        self.verbosity_level = verbosity_level
    



class Command:
    def set(self, uid, module_id, name, code, programming_language):
        '''
        Browse the database for existing command.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        command_in_db = self.cursor.execute(self.query_list.select_command.text, (uid, module_id, name, code, programming_language)).fetchone()
        if command_in_db is not None:
            self.uid = command_in_db[0]
            self.module_id = command_in_db[1]
            self.name = command_in_db[2]
            self.code = command_in_db[3]
            self.programming_language = command_in_db[4]
            self.definition = command_in_db[5]
            self.script_url = command_in_db[6]
            self.description = command_in_db[7]
            self.active = command_in_db[8]
            self.output_control.print(self.output_control.DATA_SET, ('Command ', 'ID: ', self.uid, ', Name: ', self.name, ', Code: ', self.code))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Command ', 'ID: ', self.uid, ', Name: ', self.name, ', Code: ', self.code))

    def insert(self):
        '''
        Browse the database for existing command.
        1) If not found, saves the command to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        command_in_db = self.cursor.execute(self.query_list.select_command.text, ('%', self.module_id, self.name, self.code, self.programming_language)).fetchone()
        if command_in_db is None:
            self.cursor.execute(self.query_list.insert_command.text, (self.module_id, self.name, self.code, self.programming_language, self.definition, self.script_url, self.description, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Command ', 'ID: ', self.uid, ', Name: ', self.name, ', Code: ', self.code))
        else:
            self.uid = command_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Command ', 'ID: ', self.uid, ', Name: ', self.name, ', Code: ', self.code))

    def __init__(self, query_list = None, cursor = None, uid = '%', module_id = '%', name = '%', code = '%', programming_language='%', definition = '', script_url = '', description = '', active = True, output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.module_id = module_id
        self.name = name
        self.code = code
        self.programming_language = programming_language
        self.definition = definition
        self.script_url = script_url
        self.description = description
        self.active = active
        self.output_control = output_control



import sqlite3
import subprocess
#import requests
class Executor:
    returned_args=()

    def __init__(self, output_control, database):
        self.database = database
        self.output_control = output_control

    def install_external_module(self, external_module):
        try:
            if requests.get('http://pypi.python.org/pypi/{0}/json'.format(external_module)).status_code == 200:
                #Prompt for installation
                pass
            else:
                self.output_control.print(self.output_control.EXT_MOD_NOT_FOUND, (external_module,))
        except TimeoutError as e:
            self.output_control.print(self.output_control.EXT_MOD_NOT_INSTALL, (str(e),))


    def import_external_modules(self, command_id):
        external_modules = (self.database.cursor.execute(self.database.query_list.select_external_modules_by_command_id.text, (str(command_id),))).fetchall()
        oc = self.output_control
        for external_module in external_modules:
            oc.print(oc.MOD_ATT_IMPORT, external_module)
            try:
                exec('import {0}'.format(external_module[0]))
                oc.print(oc.MOD_IMPORT, external_module)
            except ImportError as e:
                oc.print(oc.MOD_NOT_IMPORT, (external_module, str(e)))
                if internet_mode:
                    install_external_module(external_module)

    def execute_command(self, command_id, command_args):
        response_dict={}
        oc = self.output_control
        definition = 'No command'
        external_modules = ()

        command = (self.database.cursor.execute(self.database.query_list.select_command_by_id.text, (str(command_id),))).fetchone()

        if command is not None:
            programming_language = command[1]
            definition = command[2]
            #try:
            if True:
                if programming_language == 'python3':
                    self.import_external_modules(command_id)
                    exec(definition.format(*command_args))
                    '''
                    for returned_arg in self.returned_args:
                        success_response = (response_dict[100] % (command_args, returned_arg))
                        oc.print(oc.SUC_RESP, (success_response,))
                    '''
                elif programming_language == 'sql':
                    pass

                elif programming_language == 'bash':
                    subprocess.Popen(definition.format(*command_args), shell = True)

                elif programming_language == 'cpp':
                    pass

                oc.print(oc.COM_EXEC)

            #except Exception as e:
            else:
                pass
                #oc.print(oc.COM_NOT_EXEC, (str(e),))



class Call:
    def set(self, uid, command_id, language_id, response, active):
        '''
        Browse the database for existing call.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        call_in_db = self.cursor.execute(self.query_list.select_call.text, (uid, command_id, language_id, response, active)).fetchone()
        if call_in_db is not None:
            self.uid = call_in_db[0]
            self.command_id = call_in_db[1]
            self.language_id = call_in_db[2]
            self.response = call_in_db[3]
            self.active = call_in_db[4]
            self.output_control.print(self.output_control.DATA_SET, ('Call ', 'ID: ', self.uid, ', Command ID: ', self.command_id,', Response: ', self.response))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Call ', 'ID: ', self.uid, ', Command ID: ', self.command_id,', Response: ', self.response))

    def insert(self):
        '''
        Browse the database for existing call.
        1) If not found, saves the call to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        call_in_db = self.cursor.execute(self.query_list.select_call.text, ('%', self.command_id, self.language_id, self.response, self.active)).fetchone()
        if call_in_db is None:
            self.cursor.execute(self.query_list.insert_call.text, (self.command_id, self.language_id, self.response, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Call ', 'ID: ', self.uid, ', Command ID: ', self.command_id,', Response: ', self.response))
        else:
            self.uid = call_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Call ', 'ID: ', self.uid, ', Command ID: ', self.command_id,',, Response: ', self.response))
            
            
    def __init__(self, query_list = None, cursor = None, uid = '%', command_id = '%', language_code = 'en-US', response = 0,  active = True, output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.command_id = command_id
        self.language_id = 1 
        lang_in_db = self.cursor.execute(query_list.select_language.text, ('%', language_code, '%', "True")).fetchone()
        if lang_in_db is not None:
            self.language_id = lang_in_db[0]
        self.response = response
        self.active = active
        self.output_control = output_control



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='peak-bot',
    version='1.0.1a1', 
    description='Lightweight, simple, speech-to-text-oriented bot.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/doriclazar/peak_bot',
    author='Lazar Doric',
    author_email='doriclazar@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Topic :: Home Automation',
        'Topic :: Office/Business',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='peak bots',
    packages=find_packages(),
    install_requires=['pyaudio', 'google-cloud-speech'],
    package_data={'peak_data': ['*.json',],},
    include_package_data=True,
    data_files=[
        ('peak_data', ['peak_bot/peak_data/library/core.json', 'peak_bot/peak_data/library/core/append_to_file.json', 'peak_bot/peak_data/library/core/write_to_file.json', 'peak_bot/peak_data/library/networking.json', 'peak_bot/peak_data/library/networking/ping.json', 'peak_bot/peak_data/library/file_system.json', 'peak_bot/peak_data/library/file_system/copy.json', 'peak_bot/peak_data/configuration/settings.json', 'peak_bot/peak_data/configuration/audio_base.json', 'peak_bot/peak_data/configuration/lang_base.json']),
    ],
    entry_points = {
        "console_scripts": ['peak-bot=peak_bot.__main__:main']
        },
    project_urls={
        'Bug Reports': 'https://github.com/doriclazar/peak_bot/issues',
        'Source': 'https://github.com/doriclazar/peak_bot/',
    },
)



class Module:
    def set(self, uid, name, code):
        '''
        Browse the database for existing module.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        module_in_db = self.cursor.execute(self.query_list.select_module.text, (uid, name, code)).fetchone()
        if module_in_db is not None:
            self.uid = module_in_db[0]
            self.name = module_in_db[1]
            self.code = module_in_db[2]
            self.description = module_in_db[3]
            self.active = True
            self.output_control.print(self.output_control.DATA_SET, ('Module','name', self.name, ', Code: ', self.code,'',''))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Module', 'name', self.name, ', Code: ', self.code,'',''))

    def insert(self):
        '''
        Browse the database for existing module.
        1) If not found, saves the module to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        module_in_db = self.cursor.execute(self.query_list.select_module.text, ('%', self.name, self.code)).fetchone()
        if module_in_db is None:
            self.cursor.execute(self.query_list.insert_module.text, (self.name, self.code, self.description, self.active))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Module', 'name', self.name, ', Code: ', self.code,'',''))
        else:
            self.uid = module_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Module', 'name', self.name, ', Code: ', self.code,'',''))

    def __init__(self, query_list = None, cursor=None, uid='%', name='%', code='%', description = '%', active = True, output_control=None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.name = name
        self.code = code
        self.description = description
        self.active = active
        self.output_control = output_control



class Combo:
    def set(self, uid, call_id, word_id, variable_length, optional, position):
        '''
        Browse the database for existing combo.
        if not found, populates variables with provided values, 
        makes them ready for insert() function.
        '''
        combo_in_db = self.cursor.execute(self.query_list.select_combo.text, (uid, call_id, word_id, position)).fetchone()
        if combo_in_db is not None:
            self.uid = combo_in_db[0]
            self.call_id = combo_in_db[1]
            self.word_id = combo_in_db[2]
            self.variable_length = combo_in_db[3]
            self.optional = combo_in_db[4]
            self.position = combo_in_db[5]
            self.output_control.print(self.output_control.DATA_SET, ('Combo ', 'Call ID: ', self.call_id, ', Word ID: ', self.word_id,', Position: ', self.position))
        else:
            self.output_control.print(self.output_control.DATA_NOT_FOUND, ('Combo ', 'Call ID: ', self.call_id, ', Word ID: ', self.word_id,', Position: ', self.position))

    def insert(self):
        '''
        Browse the database for existing combo.
        1) If not found, saves the combo to the database,
        and populates the uid variable with newly formed id in the database.
        2) If found, populates the uid variable with found id in the database. 
        '''
        combo_in_db = self.cursor.execute(self.query_list.select_combo.text, ('%', self.call_id, self.word_id, self.position)).fetchone()
        if combo_in_db is None:
            self.cursor.execute(self.query_list.insert_combo.text, (self.call_id, self.word_id, self.variable_length, self.optional, self.position))
            self.uid = self.cursor.lastrowid
            self.output_control.print(self.output_control.DATA_INSRT, ('Combo ', 'Call ID: ', self.call_id, ', Word ID: ', self.word_id,', Position: ', self.position))
        else:
            self.uid = combo_in_db[0]
            self.output_control.print(self.output_control.DATA_EXISTS, ('Combo ', 'Call ID: ', self.call_id, ', Word ID: ', self.word_id,', Position: ', self.position))

    def __init__(self, query_list = None, cursor = None, uid = '%', call_id = '%', word_id = '%', variable_length = '%', optional = '%', position = '%', output_control = None):
        self.query_list = query_list
        self.cursor = cursor
        self.uid = uid
        self.call_id = call_id
        self.word_id = word_id
        self.variable_length = variable_length
        self.optional = optional
        self.position = position
        self.output_control = output_control
